#! /usr/bin/env python
# -*- coding: utf-8 -*-

import commands
import re


re_action = re.compile(r"^\s*(\d+)\s*:\s*(\((.*?)\))\s*$")
class TimedAction(object):
    def __init__(self, plan_line):
        match = re_action.match(plan_line.lower())
        self.index = int(match.group(1))
        self.name = match.group(2)
        self.parts = match.group(3).split()
        self.start = None
        self.duration = None
    def end(self):
        if self.start is None or self.duration is None:
            return None
        else:
            return self.start + self.duration
    def __str__(self):
        return "%s: %s [%s]" % (self.start, self.name, self.duration)
    def __repr__(self):
        return "TimedAction(%r, %r, %r, %r, %r, %r)" % (
            self.index, self.name, self.parts,
            self.start, self.end(), self.duration)


def read_plan(infile):
    return [TimedAction(line.rstrip()) for line in infile]


def annotate_with_durations(plan):
    val_iter = iter(map(str.strip, commands.getoutput(
        "./validate -v domain-temp.pddl problem-temp.pddl "
        "plan-temp.soln").splitlines()))
    plan_iter = iter(plan)
    for line in val_iter:
        if line.startswith("Checking next happening"):
            action = plan_iter.next()
            assert line == "Checking next happening (time %d)" % (
                action.index), (line, action)
            if action.parts[0].startswith("update"):
                action.duration = 1
            elif action.parts[0].startswith("change"):
                action.duration = 10
            else:
                for line in val_iter:
                    if line.endswith("increase"):
                        action.duration = int(line.split()[-2])
                        break


def schedule_sequentially(plan):
    last_time = 0
    for action in plan:
        action.start = last_time
        last_time = action.end()


def schedule_almost_sequentially(plan):
    last_time = 0
    obj_busy_time = {}
    for action in plan:
        objs = action.parts[1:]
        max_busy = max([obj_busy_time.get(obj, 0) for obj in objs])
        action.start = max_busy
        for obj in objs:
            obj_busy_time[obj] = action.end()


if __name__ == "__main__":
    import sys
    plan = read_plan(sys.stdin)
    annotate_with_durations(plan)
    schedule_almost_sequentially(plan)
    for action in sorted(plan, key=lambda action:action.start):
        print action
